@@ -11,22 +11,27 @@ use strict;
 use warnings;
 
 package Dist::Zilla::Plugin::Git::Push;
-BEGIN {
-  $Dist::Zilla::Plugin::Git::Push::VERSION = '1.111460';
+{
+  $Dist::Zilla::Plugin::Git::Push::VERSION = '2.009';
 }
 # ABSTRACT: push current branch
 
-use Git::Wrapper;
 use Moose;
 use MooseX::Has::Sugar;
 use MooseX::Types::Moose qw{ ArrayRef Str };
 
+use namespace::autoclean;
+
+with 'Dist::Zilla::Role::BeforeRelease';
 with 'Dist::Zilla::Role::AfterRelease';
+with 'Dist::Zilla::Role::Git::Repo';
 
 sub mvp_multivalue_args { qw(push_to) }
 
 # -- attributes
 
+has remotes_must_exist => ( ro, isa=>'Bool', default=>1 );
+
 has push_to => (
   is   => 'ro',
   isa  => 'ArrayRef[Str]',
@@ -35,12 +40,37 @@ has push_to => (
 );
 
 
+sub before_release {
+    my $self = shift;
+
+    return unless $self->remotes_must_exist;
+
+    my %valid_remote = map { $_ => 1 } $self->git->remote;
+    my @bad_remotes;
+
+    # Make sure the remotes we'll be pushing to exist
+    for my $remote_spec ( @{ $self->push_to } ) {
+      (my $remote = $remote_spec) =~ s/\s.*//s; # Discard branch (if specified)
+      if ($remote =~ m![:/]!) {
+        # Appears to be a URL or path, don't check it
+        $self->log("Will push to $remote (not checked)");
+      } else {
+        # Named remotes must exist
+        push @bad_remotes, $remote unless $valid_remote{$remote};
+      }
+    }
+
+    $self->log_fatal("These remotes do not exist: @bad_remotes")
+        if @bad_remotes;
+}
+
+
 sub after_release {
     my $self = shift;
-    my $git  = Git::Wrapper->new('.');
+    my $git  = $self->git;
 
     # push everything on remote branch
-    for my $remote ( @{ $self->push_to } ) { 
+    for my $remote ( @{ $self->push_to } ) {
       $self->log("pushing to $remote");
       my @remote = split(/\s+/,$remote);
       $self->log_debug($_) for $git->push( @remote );
@@ -50,6 +80,7 @@ sub after_release {
 
 1;
 
+__END__
 
 =pod
 
@@ -59,15 +90,16 @@ Dist::Zilla::Plugin::Git::Push - push current branch
 
 =head1 VERSION
 
-version 1.111460
+version 2.009
 
 =head1 SYNOPSIS
 
 In your F<dist.ini>:
 
     [Git::Push]
-    push_to = origin      ; this is the default
+    push_to = origin       ; this is the default
     push_to = origin HEAD:refs/heads/released ; also push to released branch
+    remotes_must_exist = 1 ; this is the default
 
 =head1 DESCRIPTION
 
@@ -78,14 +110,23 @@ The plugin accepts the following options:
 
 =over 4
 
-=item * 
+=item *
 
 push_to - the name of the a remote to push to. The default is F<origin>.
 This may be specified multiple times to push to multiple repositories.
 
+=item *
+
+remotes_must_exist - if true, then Git::Push checks before a release
+to ensure that all named remotes specified in C<push_to> are
+configured in your repo.  The default is true.  Remotes specified as a
+URL or path are not checked, but will produce a
+C<Will push to %s (not checked)> message.
+
 =back
 
 =for Pod::Coverage after_release
+    before_release
     mvp_multivalue_args
 
 =head1 AUTHOR
@@ -100,7 +141,3 @@ This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
-
-__END__
-
