@@ -12,32 +12,49 @@ use warnings;
 
 package Dist::Zilla::Role::Git::DirtyFiles;
 {
-  $Dist::Zilla::Role::Git::DirtyFiles::VERSION = '1.120370';
+  $Dist::Zilla::Role::Git::DirtyFiles::VERSION = '2.009';
 }
 # ABSTRACT: provide the allow_dirty & changelog attributes
 
 use Moose::Role;
 use Moose::Autobox;
-use MooseX::Has::Sugar;
-use MooseX::Types::Moose qw{ ArrayRef Str };
+use MooseX::Types::Moose qw{ ArrayRef Str RegexpRef };
+use Moose::Util::TypeConstraints;
 
+use namespace::autoclean;
 use List::Util 'first';
+use Path::Class;
+use Try::Tiny;
+
+requires qw(log_fatal repo_root zilla);
 
 # -- attributes
 
 
 has allow_dirty => (
-  ro, lazy,
+  is => 'ro', lazy => 1,
   isa     => ArrayRef[Str],
   builder => '_build_allow_dirty',
 );
-has changelog => ( ro, isa=>Str, default => 'Changes' );
+has changelog => ( is => 'ro', isa=>Str, default => 'Changes' );
+
+{
+  my $type = subtype as ArrayRef[RegexpRef];
+  coerce $type, from ArrayRef[Str], via { [map { qr/$_/ } @$_] };
+  has allow_dirty_match => (
+    is => 'ro',
+    lazy => 1,
+    coerce => 1,
+    isa => $type,
+    default => sub { [] },
+  );
+}
 
 around mvp_multivalue_args => sub {
   my ($orig, $self) = @_;
 
   my @start = $self->$orig;
-  return (@start, 'allow_dirty');
+  return (@start, 'allow_dirty', 'allow_dirty_match');
 };
 
 # -- builders & initializers
@@ -51,23 +68,41 @@ sub list_dirty_files
 {
   my ($self, $git, $listAllowed) = @_;
 
-  my @allowed = map { qr/${_}$/ } $self->allow_dirty->flatten;
+  my $git_root  = $self->repo_root;
+  my @filenames = $self->allow_dirty->flatten;
 
-  return grep { 
-      my $file = $_; 
-      if ( first { $file =~ $_ } @allowed ) { 
-          $listAllowed 
-      } else { 
-          !$listAllowed 
-      } 
-  } $git->ls_files( { modified=>1, deleted=>1 } );
+  if ($git_root ne '.') {
+    # Interpret allow_dirty relative to the dzil root
+    my $dzil_root = $self->zilla->root->absolute->resolve;
+    $git_root     = dir($git_root)
+                      ->absolute($dzil_root)
+                      ->resolve;
+
+    $self->log_fatal("Dzil root $dzil_root is not inside Git root $git_root")
+        unless $git_root->subsumes($dzil_root);
+
+    for my $fn (@filenames) {
+      try {
+        $fn = file($fn)
+                ->absolute($dzil_root)
+                ->resolve            # process ..
+                ->relative($git_root)
+                ->as_foreign('Unix') # Git always uses Unix-style paths
+                ->stringify;
+      };
+    }
+  } # end if git root ne dzil root
+
+  my $allowed = join '|', $self->allow_dirty_match->flatten, map { qr{^\Q$_\E$} } @filenames;
+
+  return grep { /$allowed/ ? $listAllowed : !$listAllowed }
+      $git->ls_files( { modified=>1, deleted=>1 } );
 } # end list_dirty_files
 
 
-no Moose::Role;
-no MooseX::Has::Sugar;
 1;
 
+__END__
 
 =pod
 
@@ -77,7 +112,7 @@ Dist::Zilla::Role::Git::DirtyFiles - provide the allow_dirty & changelog attribu
 
 =head1 VERSION
 
-version 1.120370
+version 2.009
 
 =head1 DESCRIPTION
 
@@ -92,6 +127,9 @@ A list of files that are allowed to be dirty in the git checkout.
 Defaults to C<dist.ini> and the changelog (as defined per the
 C<changelog> attribute.
 
+If your C<repo_root> is not the default (C<.>), then these filenames
+are relative to Dist::Zilla's root directory, not the Git root directory.
+
 =head2 changelog
 
 The name of the changelog. Defaults to C<Changes>.
@@ -123,7 +161,3 @@ This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
-
-__END__
-
